==> dasu_2.02.jar: edu/northwestern/dasu/stats/RemotePeerStatStatic.java <==
/**
 * Dasu Project
 *
 * File:         StaticPeerStat.java
 * RCS:          $Id: RemotePeerStatStatic.java,v 1.2 2010/11/01 15:37:00 mas939 Exp $
 * Description:  StaticPeerStat class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Oct 17, 2010 at 7:57:16 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.stats
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.stats;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;

import org.gudy.azureus2.core3.download.DownloadManager;
import org.gudy.azureus2.core3.peer.PEPeer;
import org.gudy.azureus2.plugins.download.DownloadException;
import org.gudy.azureus2.plugins.peers.Peer;
import org.gudy.azureus2.plugins.peers.PeerEvent;
import org.gudy.azureus2.plugins.peers.PeerListener2;
import org.gudy.azureus2.plugins.peers.PeerStats;

import edu.northwestern.dasu.DasuConfiguration;
import edu.northwestern.dasu.DasuManager;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.MainGeneric;
import edu.northwestern.dasu.drools.FactMeasurementRpss;
import edu.northwestern.dasu.measurement.AMCoordinatorV2;
import edu.northwestern.dasu.measurement.ProbeType;
import edu.northwestern.dasu.measurement.SourceType;
import edu.northwestern.dasu.util.PingManager;
import edu.northwestern.dasu.util.Util;
import edu.northwestern.dasu.util.Util.PingResponse;


/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The StaticPeerStat class ...
 */

public class RemotePeerStatStatic implements Cloneable {
	public Peer peer;
	public int peerId;
	public long currentLifetime;
	public ArrayList<RemotePeerStatDynamic> dynamicStats = new ArrayList<RemotePeerStatDynamic>();
	public int tcpPort;
	public int udpPort;
	public boolean sameCluster = false;
	public long firstSeen;
	public long lastTraceroute = -1;
	public int peerClient;
	public boolean done = false;
	public String encryption;
	public int type = -1;
	public long pieceSize;
	
	public boolean DEBUG = false;
	
	public RemotePeerStatStatic(Peer p) {
		this.peer = p;
		this.peerClient = -1;
		this.peerId = -1;
		this.currentLifetime = p.getStats().getTimeSinceConnectionEstablished();
		this.firstSeen = Util.currentGMTTime() - currentLifetime;			
		this.tcpPort = p.getTCPListenPort();
		this.udpPort = p.getUDPListenPort();
		
		try {
			
			Iterator<DownloadManager> iter = Main.getGlobalManager().getDownloadManagers().iterator();
			while (iter.hasNext()) {
				DownloadManager dm = iter.next();

					if (dm.getTorrentFileName().equals(peer.getManager().getDownload().getTorrentFileName())) {
						ArrayList<PEPeer> peerList = (ArrayList<PEPeer>) dm.getPeerManager().getPeers(peer.getIp());
						for ( PEPeer pe : peerList ) {
							encryption = pe.getEncryption();
							type = pe.getPeerSource().equals("Incoming")?1:0;
							pieceSize = dm.getTorrent().getPieceLength();
						}
					}
			}
			
		} catch (DownloadException e) {
			e.printStackTrace();
		}

		final RemotePeerStatStatic rpss = this;
		
		p.addListener(new PeerListener2 (){

			public void eventOccurred(PeerEvent event) {
				if (event.getType()==PeerEvent.ET_STATE_CHANGED){
					if (peer.getState()==Peer.DISCONNECTED){
						currentLifetime = Util.currentGMTTime()-firstSeen;
						
						if (rpss.peer.getStats().getTotalReceived() >0 || rpss.peer.getStats().getTotalSent() >0)
							if (DasuManager.engine != null) 
								DasuManager.engine.addFact(new FactMeasurementRpss(rpss));
							
						stop();
					}
				}
				
			}});
		
		if (DasuConfiguration.getInstance().isDoTraceroute()
				&& !MainGeneric.isShuttingDown()){
			AMCoordinatorV2.addLowPriorityProbe(p.getIp(), ProbeType.TRACEROUTE, SourceType.NULL);
		}

	}

	public void updateStats(){
		
		PeerStats ps = peer.getStats();
		final RemotePeerStatDynamic stat = new RemotePeerStatDynamic();
		stat.bytesSentToPeer = (int) ps.getTotalSent();
		stat.bytesReceivedFromPeer = (int) ps.getTotalReceived();
		stat.currentSendRate = ps.getUploadAverage();
		stat.currentReceiveRate = ps.getDownloadAverage();
		stat.isSeed = peer.isSeed();
		stat.rttPending = true;
		stat.isInterested = peer.isInterested();
		stat.isInteresting = peer.isInteresting();
				
		if (!MainGeneric.isShuttingDown()) 
			PingManager.getInstance().doPing(peer.getIp(), MainGeneric.getPluginInterface(), new PingResponse(){

			public void response(double rtt) {
				if (rtt==-1){
					//System.err.println("Ping failed to " + peer.getIp() + "!");						
				} else {
					//System.out.println("Ping succeeded!");
				}
				stat.ping = rtt;
				stat.rttPending = false;
				
			}});
		
		if (dynamicStats.size() > PeerStatistics.getInstance().MAX_DPS_ENTRIES){
			flush();
		}
		synchronized (dynamicStats){
			if (dynamicStats.size() > PeerStatistics.getInstance().MAX_DPS_ENTRIES) dynamicStats.remove(0);
			
			if (dynamicStats.size() == 0 )
				dynamicStats.add(stat);
			
			if ( !dynamicStats.get(dynamicStats.size()-1).equals(stat) )
				dynamicStats.add(stat);
			else {
				dynamicStats.get(dynamicStats.size()-1).currentTime = stat.currentTime;
				
			if (DEBUG)
				System.out.println("equal to the previous, just skip!! " + peer.getIp() + " " + stat);
			}
		}
	}

	public void setSameCluster(boolean sc){
		this.sameCluster = sc;
	}

	/**
	 * Report dynamic data.
	 *
	 */
	public void flush() {
		if (peerId == -1 ){
			peerId = Statistics.getInstance().getPeerId(peer.getIp());
		}
		
		//Statistics.getInstance().batchProcessIds();
		
		synchronized (dynamicStats){
			LinkedList<RemotePeerStatDynamic> toKeep = new LinkedList<RemotePeerStatDynamic>();
			for (RemotePeerStatDynamic dps : dynamicStats){
				if (dps.rttPending){
					toKeep.add(dps);
					continue;
				} 
				if (DasuConfiguration.getInstance().isRecordStats()){
					MainGeneric.getReporter().insertRemotePeerStatDynamic(
							Statistics.getInstance().getSelfId(), 
							peerId, dps.currentTime, dps.bytesReceivedFromPeer, 
							dps.bytesSentToPeer, dps.currentReceiveRate, 
							dps.currentSendRate, dps.isSeed, dps.ping, dps.isInterested, dps.isInteresting);
				}
			}
			
			dynamicStats.clear();
			dynamicStats.addAll(toKeep);
		}
		
	}

	/**
	 * Report once-per-session data.
	 *
	 */
	public void stop() {
		done  = true;			
	}

	public boolean isDone() {		
		return done || 
			peer.getState() == Peer.DISCONNECTED || 
			peer.getState()== Peer.CLOSING;
	}

}

==> dasu_3.02.jar: edu/northwestern/dasu/stats/RemotePeerStatStatic.java <==
/**
 * Dasu Project
 *
 * File:         StaticPeerStat.java
 * RCS:          $Id$
 * Description:  StaticPeerStat class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Oct 17, 2010 at 7:57:16 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.stats
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.stats;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;

import org.gudy.azureus2.core3.download.DownloadManager;
import org.gudy.azureus2.core3.peer.PEPeer;
import org.gudy.azureus2.plugins.download.DownloadException;
import org.gudy.azureus2.plugins.peers.Peer;
import org.gudy.azureus2.plugins.peers.PeerEvent;
import org.gudy.azureus2.plugins.peers.PeerListener2;
import org.gudy.azureus2.plugins.peers.PeerStats;

import edu.northwestern.dasu.DasuConfiguration;
import edu.northwestern.dasu.DasuManager;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.drools.FactMeasurementRpss;
import edu.northwestern.dasu.measurement.AMCoordinatorV2;
import edu.northwestern.dasu.measurement.ProbeType;
import edu.northwestern.dasu.measurement.SourceType;
import edu.northwestern.dasu.util.PingManager;
import edu.northwestern.dasu.util.Util;
import edu.northwestern.dasu.util.Util.PingResponse;


/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The StaticPeerStat class ...
 */

public class RemotePeerStatStatic implements Cloneable {
	public Peer peer;
	public int peerId;
	public long currentLifetime;
	public ArrayList<RemotePeerStatDynamic> dynamicStats = new ArrayList<RemotePeerStatDynamic>();
	public int tcpPort;
	public int udpPort;
	public boolean sameCluster = false;
	public long firstSeen;
	public long lastTraceroute = -1;
	public int peerClient;
	public boolean done = false;
	public String encryption;
	public int type = -1;
	public long pieceSize;
	
	public boolean DEBUG = false;
	
	public RemotePeerStatStatic(Peer p) {
		this.peer = p;
		this.peerClient = -1;
		this.peerId = -1;
		this.currentLifetime = p.getStats().getTimeSinceConnectionEstablished();
		this.firstSeen = Util.currentGMTTime() - currentLifetime;			
		this.tcpPort = p.getTCPListenPort();
		this.udpPort = p.getUDPListenPort();
		
		try {
			
			Iterator<DownloadManager> iter = Main.getGlobalManager().getDownloadManagers().iterator();
			while (iter.hasNext()) {
				DownloadManager dm = iter.next();

					if (dm.getTorrentFileName().equals(peer.getManager().getDownload().getTorrentFileName())) {
						ArrayList<PEPeer> peerList = (ArrayList<PEPeer>) dm.getPeerManager().getPeers(peer.getIp());
						for ( PEPeer pe : peerList ) {
							encryption = pe.getEncryption();
							type = pe.getPeerSource().equals("Incoming")?1:0;
							pieceSize = dm.getTorrent().getPieceLength();
						}
					}
			}
			
		} catch (DownloadException e) {
			e.printStackTrace();
		}

		final RemotePeerStatStatic rpss = this;
		
		p.addListener(new PeerListener2 (){

			public void eventOccurred(PeerEvent event) {
				if (event.getType()==PeerEvent.ET_STATE_CHANGED){
					if (peer.getState()==Peer.DISCONNECTED){
						currentLifetime = Util.currentGMTTime()-firstSeen;
						
						if (rpss.peer.getStats().getTotalReceived() >0 || rpss.peer.getStats().getTotalSent() >0)
							if (DasuManager.engine != null) 
								DasuManager.engine.addFact(new FactMeasurementRpss(rpss));
							
						stop();
					}
				}
				
			}});
		
		if (DasuConfiguration.getInstance().isDoTraceroute()
				&& !Main.isShuttingDown()){
			AMCoordinatorV2.addLowPriorityProbe(p.getIp(), ProbeType.TRACEROUTE, SourceType.NULL);
		}

	}

	public void updateStats(){
		
		PeerStats ps = peer.getStats();
		final RemotePeerStatDynamic stat = new RemotePeerStatDynamic();
		stat.bytesSentToPeer = (int) ps.getTotalSent();
		stat.bytesReceivedFromPeer = (int) ps.getTotalReceived();
		stat.currentSendRate = ps.getUploadAverage();
		stat.currentReceiveRate = ps.getDownloadAverage();
		stat.isSeed = peer.isSeed();
		stat.rttPending = true;
		stat.isInterested = peer.isInterested();
		stat.isInteresting = peer.isInteresting();
				
		if (!Main.isShuttingDown()) 
			PingManager.getInstance().doPing(peer.getIp(), Main.getPluginInterface(), new PingResponse(){

			public void response(double rtt) {
				if (rtt==-1){
					//System.err.println("Ping failed to " + peer.getIp() + "!");						
				} else {
					//System.out.println("Ping succeeded!");
				}
				stat.ping = rtt;
				stat.rttPending = false;
				
			}});
		
		PeerStatistics.getInstance();
		if (dynamicStats.size() > PeerStatistics.MAX_DPS_ENTRIES){
			flush();
		}
		synchronized (dynamicStats){
			PeerStatistics.getInstance();
			if (dynamicStats.size() > PeerStatistics.MAX_DPS_ENTRIES) dynamicStats.remove(0);
			
			if (dynamicStats.size() == 0 )
				dynamicStats.add(stat);
			
			if ( !dynamicStats.get(dynamicStats.size()-1).equals(stat) )
				dynamicStats.add(stat);
			else {
				dynamicStats.get(dynamicStats.size()-1).currentTime = stat.currentTime;
				
			if (DEBUG)
				System.out.println("equal to the previous, just skip!! " + peer.getIp() + " " + stat);
			}
		}
	}

	public void setSameCluster(boolean sc){
		this.sameCluster = sc;
	}

	/**
	 * Report dynamic data.
	 *
	 */
	public void flush() {
		if (peerId == -1 ){
			peerId = Statistics.getInstance().getPeerId(peer.getIp());
		}
		
		//Statistics.getInstance().batchProcessIds();
		
		synchronized (dynamicStats){
			LinkedList<RemotePeerStatDynamic> toKeep = new LinkedList<RemotePeerStatDynamic>();
			for (RemotePeerStatDynamic dps : dynamicStats){
				if (dps.rttPending){
					toKeep.add(dps);
					continue;
				} 
				if (DasuConfiguration.getInstance().isRecordStats()){
					Main.getReporter().insertRemotePeerStatDynamic(
							Statistics.getInstance().getSelfId(), 
							peerId, dps.currentTime, dps.bytesReceivedFromPeer, 
							dps.bytesSentToPeer, dps.currentReceiveRate, 
							dps.currentSendRate, dps.isSeed, dps.ping, dps.isInterested, dps.isInteresting);
				}
			}
			
			dynamicStats.clear();
			dynamicStats.addAll(toKeep);
		}
		
	}

	/**
	 * Report once-per-session data.
	 *
	 */
	public void stop() {
		done  = true;			
	}

	public boolean isDone() {		
		return done || 
			peer.getState() == Peer.DISCONNECTED || 
			peer.getState()== Peer.CLOSING;
	}

}
